﻿using System.Collections.Generic;
using System.Data.Entity;
using System.ComponentModel.DataAnnotations;

namespace MyDal
{
    public class Member
    {
        public int ID { get; set; }
        [Required(ErrorMessage = "名前は必須入力です")]
        public string Name { get; set; }
        [RegularExpression(@"[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,4}", ErrorMessage = "メールが正しくありません")]
        public string Mail { get; set; }
        [Required(ErrorMessage = "カテゴリは必須入力です")]
        public int CategoryID { get; set; }

        public virtual Category Category { get; set; }
    }

    public class Category
    {
        public int CategoryID { get; set; }
        public string Name { get; set; }

        public virtual ICollection<Member> Members { get; set; }
    }

    public class MemberDB : DbContext
    {
        public DbSet<Member> Members { get; set; }
        public DbSet<Category> Categories { get; set; }

    }

    public class GroupInitializer : DropCreateDatabaseIfModelChanges<MemberDB>
    {
        protected override void Seed(MemberDB context)
        {
            var categories = new List<Category>
            {
                new Category{ Name = "シルバー会員"},
                new Category{ Name = "一般会員"},
                new Category{ Name = "ジュニア会員"}
            };
            categories.ForEach(s => context.Categories.Add(s));
            context.SaveChanges();
        }
    }
}